#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.Indicators;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Strategies in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Strategies
{
	
	[Gui.CategoryOrder("Money-Management", 1)]
	[Gui.CategoryOrder("Bollinger", 2)]
	[Gui.CategoryOrder("Auszeit 1", 3)]
	[Gui.CategoryOrder("Auszeit 2", 4)]
	[Gui.CategoryOrder("Auszeit 3", 5)]
	
	public class BollingerEURUSD : Strategy
	{
		private Bollinger Bollinger1;
		private Bollinger Bollinger2;
		
		private SMA SMA1;
	
		public enum Options_Weekdays : int {
			alle = 0, Montag = 1, Dienstag = 2, Mittwoch = 3, Donnerstag = 4, Freitag = 5
		}
		
		private Boolean LastFilledOrderWasAStopLoss = false;

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Enter the description for your new custom Strategy here.";
				Name										= "BollingerEURUSD";
				Calculate									= Calculate.OnBarClose;
				EntriesPerDirection							= 1;
				EntryHandling								= EntryHandling.AllEntries;
				IsExitOnSessionCloseStrategy				= false;
				ExitOnSessionCloseSeconds					= 30;
				IsFillLimitOnTouch							= false;
				MaximumBarsLookBack							= MaximumBarsLookBack.TwoHundredFiftySix;
				OrderFillResolution							= OrderFillResolution.Standard;
				Slippage									= 0;
				StartBehavior								= StartBehavior.WaitUntilFlat;
				TimeInForce									= TimeInForce.Gtc;
				TraceOrders									= false;
				RealtimeErrorHandling						= RealtimeErrorHandling.IgnoreAllErrors;
				StopTargetHandling							= StopTargetHandling.PerEntryExecution;
				BarsRequiredToTrade							= 20;
				IncludeCommission                           = true;

				IsInstantiatedOnEachOptimizationIteration	= true;
				
				EntryBollDev    = 2.2;
				EntryBollPeriod = 16;
				ExitBollDev     = 2.9;
				ExitBollPeriod  = 21;
				
				// in percent
				StopLossValue   = 0.8;
				TakeProfitValue = 0.6;
			
				LimitBuyValue = 1.0;
				
				Weekday = 0;
				
				TimeOut_1_Start = DateTime.Parse( "14:55" );
				TimeOut_1_End   = DateTime.Parse( "15:05" );

				TimeOut_2_Start = DateTime.Parse( "08:55" );
				TimeOut_2_End   = DateTime.Parse( "13:00" );

				TimeOut_3_Start = DateTime.Parse( "00:00" );
				TimeOut_3_End   = DateTime.Parse( "00:00" );
				
				SMAPeriod = 67;
				
			}
			else if ( State == State.Configure ) {
			}
			else if ( State == State.DataLoaded ) {
				
				Bollinger1 = Bollinger(Close, ExitBollDev, Convert.ToInt32(ExitBollPeriod));
				Bollinger2 = Bollinger(Close, EntryBollDev, Convert.ToInt32(EntryBollPeriod));
				
				// Exit
				Bollinger1.Plots[0].Brush = Brushes.Red;
				Bollinger1.Plots[1].Brush = Brushes.Transparent;
				Bollinger1.Plots[2].Brush = Brushes.Transparent;
				AddChartIndicator( Bollinger1 );

				// Entry
				Bollinger2.Plots[0].Brush = Brushes.Transparent;
				Bollinger2.Plots[1].Brush = Brushes.Transparent;
				Bollinger2.Plots[2].Brush = Brushes.Green;
				AddChartIndicator( Bollinger2 );
				
				//
				SMA1 = SMA( SMAPeriod );
				AddChartIndicator( SMA1 );

			}
		}

		protected override void OnBarUpdate()
		{
			if ( BarsInProgress != 0 ) 
				return;

			if ( CurrentBars[0] < 100 )
				return;

			if ( High[0] > Bollinger1.Upper[0] ) {
				ExitLong( Convert.ToInt32( DefaultQuantity ), "", "" );
			}
			
			if ( Position.MarketPosition == MarketPosition.Long ) {
				return;
			}
			
			if ( LastFilledOrderWasAStopLoss && High[0] > Bollinger1.Upper[0] ) {
				LastFilledOrderWasAStopLoss = false;
			}
			
			bool Do_The_Trade = false;
			if ( Weekday == Options_Weekdays.alle ) {
				Do_The_Trade = true;
			} else if ( Weekday == Options_Weekdays.Montag ) {
				if ( Time[0].DayOfWeek == DayOfWeek.Monday ) {
					Do_The_Trade = true;
				}
			} else if ( Weekday == Options_Weekdays.Dienstag ) {
				if ( Time[0].DayOfWeek == DayOfWeek.Tuesday ) {
					Do_The_Trade = true;
				}
			} else if ( Weekday == Options_Weekdays.Mittwoch ) {
				if ( Time[0].DayOfWeek == DayOfWeek.Wednesday ) {
					Do_The_Trade = true;
				}
			} else if ( Weekday == Options_Weekdays.Donnerstag ) {
				if ( Time[0].DayOfWeek == DayOfWeek.Thursday ) {
					Do_The_Trade = true;
				}
			} else if ( Weekday == Options_Weekdays.Freitag ) {
				if ( Time[0].DayOfWeek == DayOfWeek.Friday ) {
					Do_The_Trade = true;
				}
			}
			
		
			if ( TimeOut_1_Start.TimeOfDay < TimeOut_1_End.TimeOfDay && Time[0].TimeOfDay >= TimeOut_1_Start.TimeOfDay && Time[0].TimeOfDay < TimeOut_1_End.TimeOfDay ) {
				Do_The_Trade = false;
			}

			if ( TimeOut_2_Start.TimeOfDay < TimeOut_2_End.TimeOfDay && Time[0].TimeOfDay >= TimeOut_2_Start.TimeOfDay && Time[0].TimeOfDay < TimeOut_2_End.TimeOfDay ) {
				Do_The_Trade = false;
			}

			if ( TimeOut_3_Start.TimeOfDay < TimeOut_3_End.TimeOfDay && Time[0].TimeOfDay >= TimeOut_3_Start.TimeOfDay && Time[0].TimeOfDay < TimeOut_3_End.TimeOfDay ) {
				Do_The_Trade = false;
			}

			if ( Do_The_Trade && Close[1] > Bollinger2.Lower[1] && Close[0] < Bollinger2.Lower[0] && SMA1[0] > SMA1[1] && !LastFilledOrderWasAStopLoss  ) {

				
				if ( LimitBuyValue > 0 ) {
					EnterLong( Convert.ToInt32(DefaultQuantity), "" );
				} else {
					EnterLongLimit( Convert.ToInt32(DefaultQuantity), Close[0] + LimitBuyValue, "" );
				}				
				
				if ( StopLossValue > 0 ) {
					SetStopLoss( CalculationMode.Percent, StopLossValue/100 );
				}
				if ( TakeProfitValue > 0 ) {
					SetProfitTarget( CalculationMode.Percent, TakeProfitValue/100 );
				}
			}
			
			
		}
		
		protected override void OnOrderUpdate(Order order, double limitPrice, double stopPrice, int quantity, int filled, double averageFillPrice, OrderState orderState, DateTime time, ErrorCode error, string comment ) {
			
			if ( order.OrderState == OrderState.Filled ) {
				LastFilledOrderWasAStopLoss = order.Name == "Stop loss";
			}
		}

		#region Properties
		
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="SL in %", Order=0, GroupName="Money-Management")]
		public double StopLossValue
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="TP in %", Order=1, GroupName="Money-Management")]
		public double TakeProfitValue
		{ get; set; }

		[NinjaScriptProperty]
		[Range(double.MinValue, double.MaxValue)]
		[Display(Name="Kauflimit", Order=3, GroupName="Money-Management")]
		public double LimitBuyValue
		{ get; set; }

		[NinjaScriptProperty]
		[Range(int.MinValue, int.MaxValue)]
		[Display(Name="GD Periode", Order=4, GroupName="Money-Management")]
		public int SMAPeriod
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, double.MaxValue)]
		[Display(Name="EntryBollDev", Order=1, GroupName="Bollinger")]
		public double EntryBollDev
		{ get; set; }

		[NinjaScriptProperty]
		[Range(5, int.MaxValue)]
		[Display(Name="EntryBollPeriod", Order=2, GroupName="Bollinger")]
		public int EntryBollPeriod
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, double.MaxValue)]
		[Display(Name="ExitBollDev", Order=3, GroupName="Bollinger")]
		public double ExitBollDev
		{ get; set; }

		[NinjaScriptProperty]
		[Range(2, int.MaxValue)]
		[Display(Name="ExitBollPeriod", Order=4, GroupName="Bollinger")]
		public int ExitBollPeriod
		{ get; set; }
		
		[NinjaScriptProperty]
		[Display(Name="Wochentag", Order=5, GroupName="Bollinger")]
		public Options_Weekdays Weekday
		{ get; set; }
		
		[NinjaScriptProperty]
		[PropertyEditor("NinjaTrader.Gui.Tools.TimeEditorKey")]
		[Display(Name="Beginn", Order=0, GroupName="Auszeit 1")]
		public DateTime TimeOut_1_Start
		{ get; set; }

		[NinjaScriptProperty]
		[PropertyEditor("NinjaTrader.Gui.Tools.TimeEditorKey")]
		[Display(Name="Ende", Order=1, GroupName="Auszeit 1")]
		public DateTime TimeOut_1_End
		{ get; set; }
		
		[NinjaScriptProperty]
		[PropertyEditor("NinjaTrader.Gui.Tools.TimeEditorKey")]
		[Display(Name="Beginn", Order=0, GroupName="Auszeit 2")]
		public DateTime TimeOut_2_Start
		{ get; set; }

		[NinjaScriptProperty]
		[PropertyEditor("NinjaTrader.Gui.Tools.TimeEditorKey")]
		[Display(Name="Ende", Order=1, GroupName="Auszeit 2")]
		public DateTime TimeOut_2_End
		{ get; set; }
		
		[NinjaScriptProperty]
		[PropertyEditor("NinjaTrader.Gui.Tools.TimeEditorKey")]
		[Display(Name="Beginn", Order=0, GroupName="Auszeit 3")]
		public DateTime TimeOut_3_Start
		{ get; set; }

		[NinjaScriptProperty]
		[PropertyEditor("NinjaTrader.Gui.Tools.TimeEditorKey")]
		[Display(Name="Ende", Order=1, GroupName="Auszeit 3")]
		public DateTime TimeOut_3_End
		{ get; set; }
		
		#endregion

	}
}
